const util = require('util');
const fs = require('fs');
const path = require('path');

const mkdirp = util.promisify(require('mkdirp'));
const rimraf = util.promisify(require('rimraf'));

const sync = require('../index');

const ORIGINAL_DIRECTORY = "./spec/fixtures/original/";
const OUTPUT_DIRECTORY = "./spec/fixtures/outputs/";

const PATH_FOO = "foo";
const PATH_BAR = "bar";
const PATH_BAZ = "baz";
const PATH_QUX = path.join("baz", "qux");
const PATH_NEST = path.join("nest1", "nest2", "item");

describe("Synchronize", () => {

  beforeEach((done) => {
    mkdirp(OUTPUT_DIRECTORY).then(done);
  });

  afterEach((done) => {
    rimraf(OUTPUT_DIRECTORY).then(done);
  });

  it("normal", (done) => {
    return Promise
    .resolve()
    .then(() => sync(
      ORIGINAL_DIRECTORY,
      OUTPUT_DIRECTORY
    ))
    .then(() => {
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_FOO))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_BAR))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_QUX))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_NEST))).toBeTruthy();
    })
    .then(done)
  });

  it("designation glob (all)", (done) => {
    return Promise
    .resolve()
    .then(() => sync(
      ORIGINAL_DIRECTORY,
      OUTPUT_DIRECTORY,
      [
        "**/*",
      ]
    ))
    .then(() => {
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_FOO))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_BAR))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_QUX))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_NEST))).toBeTruthy();
    })
    .then(done)
  });

  it("designation glob (flat)", (done) => {
    return Promise
    .resolve()
    .then(() => sync(
      ORIGINAL_DIRECTORY,
      OUTPUT_DIRECTORY,
      [
        "*",
      ]
    ))
    .then(() => {
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_FOO))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_BAR))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_QUX))).toBeFalsy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_NEST))).toBeFalsy();
    })
    .then(done)
  });

  it("designation multiple", (done) => {
    return Promise
    .resolve()
    .then(() => sync(
      ORIGINAL_DIRECTORY,
      OUTPUT_DIRECTORY,
      [
        "foo",
        "baz/*",
      ]
    ))
    .then(() => {
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_FOO))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_BAR))).toBeFalsy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_QUX))).toBeTruthy();
      expect(fs.existsSync(path.join(OUTPUT_DIRECTORY, PATH_NEST))).toBeFalsy();
    })
    .then(done)
  });

});